#!/usr/local/appcenter-python
#
# All Rights Reserved.
# Copyright 2010-2012 Nukona, Inc
#

import argparse
import glob

from ac_common.core.script_utils import ScriptBase
from ac_common.conf import settings
from appstore.models import Tenant, TenantLicensing, TenantLicensingSN
from appstore.licensing import Licensing

from ac_common.djangoapps import xlog; logger = xlog.getLogger()


class SLFLicense(object):

    def __init__(self, slf, licensing_version, repo_path):
        self.slf = slf
        self.repo_path = repo_path
        self.licensing_version = licensing_version
        self._parsed = False
        self._xml = None

    def get_serial_number(self):
        license_dom = self.get_parsed_slf()
        if not license_dom:
            print('Failed to parse document')
            return None

        try:
            serial_number = license_dom.getElementsByTagName('serial_number')[0].firstChild.data
        except IndexError:
            serial_number = None

        if not serial_number:
            print('Failed to find serial number in SLF contents')
            return None

        return serial_number

    def get_parsed_slf(self):
        if not self._parsed:
            self._parsed = True
            self._xml = self._parse()
        return self._xml

    def _parse(self):
        from xml.dom.minidom import parseString
        return parseString(self.slf)


class Script(ScriptBase):
    DESCRIPTION = 'Update missing license information for a tenant'
    KEYWORDS = ['license', 'database']

    arg_parser = argparse.ArgumentParser(description=DESCRIPTION)
    arg_parser.add_argument('tenant_name_or_id',
                        type=str,
                        nargs='?',
                        default=None,
                        help='A tenant name or ID')

    def run(self, *raw_args):
        args = self.arg_parser.parse_args(raw_args)

        tenant = None
        try:
            tenant = Tenant.objects.get(name=args.tenant_name_or_id)
        except Tenant.DoesNotExist:
            pass
        if not tenant:
            try:
                tenant = Tenant.objects.get(id=int(args.tenant_name_or_id))
            except (Tenant.DoesNotExist, ValueError, TypeError):
                pass
        if not tenant:
            self.logger.error('Failed to find tenant "%s"' % args.tenant_name_or_id)
            return False

        print "Forcing license repo refresh for tenant: %s"%(tenant.name)
        l = Licensing(tenant)
        l.GetLicenseInfo()
        del l

        tl = TenantLicensing.objects.get(tenant=tenant)
        query = TenantLicensingSN.objects.filter(tenant_licensing=tl, license_slf='')

        print "Number of licenses with missing data: %d"%(query.count())

        if not query.count():
            return True

        serial_map = {}
        for filename in glob.glob("%s/%d/*.slf" % (settings.LICENSING_REPOSITORY_ROOT, tenant.id)):
            try:
                with open(filename) as lic_file:
                    lic_str = lic_file.read()
                lic_reader = SLFLicense(lic_str, '4.0', settings.LICENSING_REPOSITORY_ROOT)
                serial_number = lic_reader.get_serial_number()
                if serial_number:
                    serial_map[serial_number] = lic_str
                    print('File %s contains serial %s'%(filename, serial_number))
            except (IOError, OSError) as e:
                logger.warn("Failed to read file %s"%filename, data={'Exception':e})

        for tlsn in query:
            tlsn.license_slf = serial_map.get(tlsn.tenant_serial_number.serial_number, '')
            tlsn.save()
            if tlsn.license_slf:
                print("Updated license %s."%tlsn.tenant_serial_number.serial_number)

        return True
